﻿using Plane.Logging;
using GalaSoft.MvvmLight.Ioc;
using Microsoft.Practices.ServiceLocation;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;

namespace Plane.FormationCreator.Formation
{
    public class MapServer : IDisposable
    {
        private HttpListener _listner = new HttpListener();
        private Dictionary<string, Stream> _resources;
        private int _port;

        private static ILogger _logger = ServiceLocator.Current.GetInstance<ILogger>();

        public static MapServer Create(Dictionary<string, Stream> resources)
        {
            const int ORIGINAL_PORT = 10240;
            int port = ORIGINAL_PORT;
            var r = new Random();
            MapServer _earthResourcesServer = null;
            for (int i = 0; ; ++i)
            {
                _earthResourcesServer = new MapServer(resources, port);
                try
                {
                    _earthResourcesServer.Start();
                    _earthResourcesServer._port = port;
                    if (port != ORIGINAL_PORT)
                    {
                        using (var reader = new StreamReader(resources["js/earth.js"]))
                        {
                            var ms = new MemoryStream();
                            var writer = new StreamWriter(ms);
                            writer.Write(reader.ReadToEnd().Replace(ORIGINAL_PORT.ToString(), port.ToString()));
                            writer.Flush();
                            resources["js/earth.js"] = ms;  // The old one will be closed when the reader is closed.
                        }
                    }
                    return _earthResourcesServer;
                }
                catch (HttpListenerException ex)
                {
                    _logger.Log(ex);
                    if (i >= 100)
                    {
                        // Alert.Show("找不到空闲端口，建议重启电脑后再试。", "Message", MessageBoxButton.OK);
                        App.Current.Shutdown();
                    }
                    port = r.Next(ORIGINAL_PORT + 1, 65536);
                }
            }
        }

        private MapServer(Dictionary<string, Stream> resources, int port)
        {
            _resources = resources;
            _listner.Prefixes.Add("http://localhost:" + port + "/");
        }

        private string _HomePageUrl;
        public string HomePageUrl { get { return _HomePageUrl ?? (_HomePageUrl = $"http://localhost:{_port}/Cesium-1.9/Apps/HelloWorld.html"); } }

        public void Start()
        {
            _listner.Start();
            _listner.BeginGetContext(AfterGetContext, null);
        }

        private void AfterGetContext(IAsyncResult ar)
        {
            if (!_listner.IsListening) return;
            var context = _listner.EndGetContext(ar);
            _listner.BeginGetContext(AfterGetContext, null);

            var response = context.Response;
            string key = context.Request.Url.LocalPath.TrimStart('/').ToLower();
            response.ContentType =
                key.EndsWith(".html")
                ? "text/html"
                : key.EndsWith(".css")
                ? "text/css"
                : key.EndsWith(".js")
                ? "application/javascript"
                : key.EndsWith(".gltf")
                ? "application/json"
                : key.EndsWith(".jpg")
                ? "image/jpeg"
                : key.EndsWith(".png")
                ? "image/png"
                : "text/plain";
            try
            {
                if (!_resources.ContainsKey(key))
                {
                    response.StatusCode = 404;
                    response.StatusDescription = "Not Found";
                }
                else
                {
                    _resources[key].Position = 0;
                    _resources[key].CopyTo(response.OutputStream);
                    response.OutputStream.Flush();
                }
            }
            catch (Exception ex)
            {
                _logger.Log(ex);
                response.StatusCode = 500;
                response.StatusDescription = "Internal Server Error";
            }
            finally
            {
                response.Close();
            }
        }

        public void Stop()
        {
            _listner.Stop();
        }

        public void Close()
        {
            _listner.Close();
        }

        void IDisposable.Dispose()
        {
            Close();
        }
    }
}
